
% program for alternative optimization of measurement matrix and multiple dictionary
% "On Collaborative Compressive Sensing Systems: The Framework, Design and Algorithm"

% Input:
%%%%%%%%%  X - Original signal set of dimension N x P
%%%%%%%%%  Psi_0 - Initial dictionary of dimension N x L
%%%%%%%%%  alpha, beta -  weighting factors in CCS
%%%%%%%%%  K - The sparsity
%%%%%%%%%  N_ite - The number of iteration in alternative optimization
%%%%%%%%%  numPsi - the number of dictionaries in CCS

% Output:
%%%%%%%%%  Phi_opt - Optimal output measurement matrix of dimension M x N
%%%%%%%%%  Psi_opt - Optimal output numPsi dictionaries of dimension N x L

function [Phi_opt,Psi_opt]=learning_CCS(X,M,Psi_0,alpha,beta,K,N_ite,numPsi)

[N,P]=size(X); [N3,L]=size(Psi_0);

for i = 1:numPsi
    Psi(:,:,i)=Psi_0;
end
num = floor(P/numPsi); X = X(:,1:num*numPsi);
X = reshape(X,N,num,numPsi);
P = num;
[U_Psi,S_Psi,V_Psi] = svd(Psi_0); Phi = U_Psi(:,1:M)';
S = zeros(L,num,numPsi);
for i=1:N_ite
    %% version 1
    Temp = zeros(N,N,numPsi);
    B = [eye(N); sqrt(beta)*(eye(N) - Phi'*Phi);sqrt(alpha)*Phi];
    BB = B'*B;
    for ii = 1:numPsi
        A = [X(:,:,ii);zeros(N,P);sqrt(alpha)*Phi*X(:,:,ii);];
        S(:,:,ii) = OMP(B*Psi(:,:,ii), A, K );   
        for j =1:L
            tmpS=S(:,:,ii);
            tmpS(j,:)=0;
            Psi(:,j,ii)=Gen_Quadratic_Constr_LS(B'*(A - B*Psi(:,:,ii)*tmpS)*S(j,:,ii)',BB*norm(S(j,:,ii))^2);
        end;    
        PsiS = Psi(:,:,ii)*S(:,:,ii);
        Temp(:,:,ii) =  beta*PsiS*PsiS' - alpha*(X(:,:,ii)-PsiS)*(X(:,:,ii)-PsiS)';
        Psi(:,:,ii) = normc(Psi(:,:,ii));
    end
    [U,D] = eig(sum(Temp,3));
    [d, index] = sort(diag(D),'descend');
    Phi = U(:,index(1:M))';
end
%[U_Psi,S_Psi,V_Psi] = svd(Psi); Phi = U_Psi(:,1:M)';
Phi_opt=Phi;
Psi_opt=Psi;


