function [Dictionary Error] = Coupled_ISD_Dic_new_image_OMP(Data,IniDic,kappa,nIter,alpha,mu_0,Iter_inside,method,Phi,alpha_dic)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Given data\in R^{N x L}, the sparsity \kappa, the number of atoms J desired,
%            and \alpha to balance the coherence and sparse representatin
%            error,
%    we solve
%       min_{D,S} ||data - DS||_F^2 + \alpha* || D'D - H||
%          s.t. ||S(:,l)||0 \leq \kappa, ||D(:,j)||_2 = 1
% -- by LG and ZZ, Epril 2016.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%gamma = 0.95; % thresholds for the maximum coherence
[N,J] = size(IniDic);
Dictionary = IniDic;
Dictionary=Dictionary*diag(1./sqrt(sum(Dictionary.*Dictionary))); % normalization

% Sparse coding with the initial dictionary
CoefMatrix=OMP(Dictionary,Data,kappa);

%totalErr = zeros(nIter,1); DicRatio = zeros(nIter,1);
coherence = zeros(nIter,1);
% Learning loop
iter = 1;

Phibar = alpha_dic^2*eye(N) + Phi'*Phi;
while iter <= nIter
    
    %%% Sheringkaging operation, yielding solu. of min_H ||H - G||_F
    G=Dictionary'*Dictionary;
    [i,j]=find(abs(G)>mu_0);  [n1,n2]=size(i); K=n1*n2;
    H = G;
    for k=1:K
        if i(k) ~= j(k)
            H(i(k),j(k)) = sign(G(i(k),j(k)))*mu_0;
        end
    end
    %      H = eye(J);
    
    %%% Iteratively update the transform
    for ite = 1:Iter_inside
        for m = 1: J
            if method ==1
                barD = [Dictionary(:,1:m-1) Dictionary(:,m+1:end)];
                tmpD = Dictionary; tmpD(:,m) = 0;
                errors=Data-tmpD*CoefMatrix;
                a = Phibar*errors*CoefMatrix(m,:)' + 2*alpha*barD*[H(1:m-1,m); H(m+1:end,m)];
                B = CoefMatrix(m,:)*CoefMatrix(m,:)'*Phibar + 2*alpha*barD*barD';
                
                
                
                
                if norm(B)<1e-8
                    Dictionary(:,m) = randn(N,1);Dictionary(:,m) = Dictionary(:,m)/norm(Dictionary(:,m));
                else
                    Dictionary(:,m) = Quadratic_Constr_LS(B^(-1/2)*a,B^(1/2));
                end
                % Dictionary(:,m) = Gen_Quadratic_Constr_LS(a,B);
                
            elseif method ==2
                barD = [Dictionary(:,1:m-1) Dictionary(:,m+1:end)];
                relevantDataIndices = find(CoefMatrix(m,:));
                tmpCoefMatrix = CoefMatrix(:,relevantDataIndices); tmpCoefMatrix(m,:) = 0;
                errors=Data(:,relevantDataIndices)-Dictionary*tmpCoefMatrix;
                a = errors*CoefMatrix(m,relevantDataIndices)' + 2*alpha*barD*[H(1:m-1,m); H(m+1:end,m)];
                B = CoefMatrix(m,relevantDataIndices)*CoefMatrix(m,relevantDataIndices)'*eye(N) + 2*alpha*barD*barD';
                if norm(B)<1e-8
                    Dictionary(:,m) = randn(N,1);Dictionary(:,m) = Dictionary(:,m)/norm(Dictionary(:,m));
                else
                    Dictionary(:,m) = Quadratic_Constr_LS(B^(-1/2)*a,B^(1/2));
                end
                CoefMatrix(m,relevantDataIndices) = Dictionary(:,m)'*errors/norm(Dictionary(:,m))^2;    
            elseif method ==3
                barD = [Dictionary(:,1:m-1) Dictionary(:,m+1:end)];
                relevantDataIndices = find(CoefMatrix(m,:));
                tmpCoefMatrix = CoefMatrix(:,relevantDataIndices); tmpCoefMatrix(m,:) = 0;
                errors=Data(:,relevantDataIndices)-Dictionary*tmpCoefMatrix;
                a = errors*CoefMatrix(m,relevantDataIndices)' + 2*alpha*barD*[H(1:m-1,m); H(m+1:end,m)];
                B = CoefMatrix(m,relevantDataIndices)*CoefMatrix(m,relevantDataIndices)'*eye(N) + 2*alpha*barD*barD';
                if norm(B)<1e-8
                    Dictionary(:,m) = randn(N,1);Dictionary(:,m) = Dictionary(:,m)/norm(Dictionary(:,m));
                else
                    Dictionary(:,m) = Quadratic_Constr_LS(B^(-1/2)*a,B^(1/2));
                end
            end
        end
    end
    %%% checking the coherence replacing the very coherent rows
    % method 3: this is used in K-SVD toolbox. We may use this if we
    % compare with them.
    % lean-up rarely used or too-close atoms
    T2=0.99; T1=3;
    Er=sum((Data-Dictionary*CoefMatrix).^2,1);
    G=Dictionary'*Dictionary;
    G=G-diag(diag(G));
    for j=1:1:size(Dictionary,2)
        if max(G(j,:))>T2
            alternativeAtom=find(G(j,:)==max(G(j,:)));
            [val,pos]=max(Er);
            Er(pos(1))=0;
            Dictionary(:,j)=Data(:,pos(1))/norm(Data(:,pos(1)));
            % CoefMatrix(alternativeAtom,:)=CoefMatrix(j,:)+CoefMatrix(alternativeAtom,:);
            % CoefMatrix(j,:)=0;
            % CoefMatrix(j,pos(1))=1;
            G=Dictionary'*Dictionary;
            G=G-diag(diag(G));
        elseif length(find(abs(CoefMatrix(j,:))>1e-7))<=T1
            [val,pos]=max(Er);
            Er(pos(1))=0;
            Dictionary(:,j)=Data(:,pos(1))/norm(Data(:,pos(1)));
            % CoefMatrix(j,:)=0;
            % CoefMatrix(j,pos(1))=1;
            G=Dictionary'*Dictionary;
            G=G-diag(diag(G));
        end;
    end;
    %%% %%%%%%%%%%%%    
    %%% Update Sparse Coefficient: with hard thresholding for solving L-0 minimization
    CoefMatrix=OMP(Dictionary,Data,kappa);
    %%%  Compute the errors and display
 
    
    AA = Dictionary'*Dictionary - eye(J);
    coherence(iter) = max(abs(AA(:)));
    fprintf('maximum coherence  = %.2f\n', coherence(iter));
    iter = iter + 1;
end

Error = Data - Dictionary*CoefMatrix;