% This is the Coupled_KSVD code
%                   ------Simultaneous dictionary and measurement matrix

% Input: 
%       X: The original signal of NP
%       Phi: The initial measurement matrix of MN
%       Psi: The initial dictionary of NL
%       K: The sparsity of the signal
%       alpha: The weighting factor

% Output: 
%       Psi_opt: The optimal dictionary of NL


function [Psi_opt]=Coupled_KSVD(X,Phi,Psi,K,alpha)

[N,P]=size(X);
[N1,L]=size(Psi);
[M1,N2]=size(Phi);
Psi=Psi*diag(1./sqrt(diag(Psi'*Psi)));  %normalization
Y=Phi*X;  % The measurement


Z=[alpha*X;Y];
W=[alpha*eye(N);Phi];
D=W*Psi;
param=struct('K',L,'numIteration',1,'errorFlag',0,'preserveDCAtom',0, ...
    'L',K,'InitializationMethod','GivenMatrix','initialDictionary',D,'displayProgress',1);
[D_opt,output]=KSVD_elad(Z,param);
Psi_opt=pinv(alpha^2*eye(N)+Phi'*Phi)*([alpha*eye(N) Phi'])*D_opt;
%Psi_opt=(alpha^2*eye(N)+Phi'*Phi)\[alpha*eye(N) Phi']*D_opt;
Psi_opt=Psi_opt*diag(1./sqrt(diag(Psi_opt'*Psi_opt))); 

end
