function [ Phi_opt,object ] = ETF(Psi,E,M,alpha,iter)
%==========================================================================
%this function is correspond to the algorithm II,alternating minimize the
%Gt and the sensing matrix. use the shrinkage algorithm to update Gt,and
%use Theroem I if M<N  or Theroem II if M>N in our paper to update sensing
%matrix
%
%Input:    Psi,the given dictionary, N*L
%          E,the traning error matrix
%          M,the dimension of the sensing matrix
%          alpha,the lagrange multipler used in our paper
%   (the first five input arguments is used in the function"closed_form_SMOSRE")
%          iter,the prescribed iterative number in algorithm II
%
%Output:   Phi_opt,the optimal sensing matrix designed by our proposed
%          algorithm,M*N
%
%                        Xiao Li NOV. 2014
%==========================================================================

%=========================================================================
[N,L] = size(Psi); 
epsilon = sqrt((L-M)/(M*(L-1)))*1; %calculate the optimal thresholding displayed  in our paper

[U_psi,Lambda_psi,V_psi] = svd(Psi);
%r = rank(Psi);
Lambda_psi = Lambda_psi(1:N,1:N);
Phi = eye(M)*[eye(M) zeros(M,N-M)]*pinv(Lambda_psi)*U_psi';  % the warm initial with (13) in our paper which is discussed in [14],what should be pointed out is that any arbitrary
% Phi = randn(M,N);
%initial sensing matrix can lead to the same performance result.
%==========================================================================

for k=1:iter
 
    %update Gt use the  shrinkage algorithm with (27) in our paper
    Gt = Psi' * Phi' * Phi * Psi;
    pos = find( abs(Gt)>epsilon );
    Gt(pos) = sign( Gt(pos) ) * epsilon;
    Gt = Gt - diag(diag(Gt)) + eye(L);


    %update Phi
    Phi = closed_form_SMOSRE(Psi,E,M,alpha,Gt);
    
%     %some stop criterion 
    object(k) = ( norm(Psi' * Phi' * Phi * Psi-Gt,'fro') )^2 + alpha * ( norm(Phi*E,'fro') )^2;
%     if(k>1)
%              if(object(k)>=object(k-1))
%                  display('the algorithm terminated');
%                  display(['the iterative number is k= ' num2str(k-1)])
%                   Phi_opt=Phi1;
%                  return 
%              end
%     end
%      Phi1=Phi;
end
          
    Phi_opt=Phi;
         
end





