%=============================================
% For given a and B, solve 
%     min_x ||a - B x||^2_2  subject to ||x||_2=1 

% =============================================

function [x_opt]=Quadratic_Constr_LS(a,B) 

%  Inputs --      A - L x 1        B - L x N 
 [L,N] = size(B);    
 [U,Sigma,~]=svds(B,min(L,N));
 
 if max(abs(a)) < 1e-6
    [value,index] = min(diag(Sigma));
    x_opt = U(:,index);
    return
else
 index = find(diag(Sigma)>1e-6); % ignore too small singular values and 0
 
 epsilon = (abs(U(:,index)'*a)).^2; % this is the epsilon^2 of the paper
 sigma = (diag(Sigma(index,index))).^2; % this is the sigma^2 of the paper
 clear U Sigma index
 
 myfun = @(lambda) sum(epsilon.*sigma./(lambda + sigma).^2)-1;
 
 lambda_up = sqrt(epsilon'*sigma); % the upper bound
 lambda_lower = max(-sigma + sqrt(epsilon.*sigma)); % the lower bound
 
 lambda_opt = fzero(myfun,[lambda_lower lambda_up]); % finding the larget root

 x_opt = pinv(lambda_opt*eye(N) + B'*B)*B'*a;
 end
 
     
                                   