
% program for alternative optimization of measurement matrix and dictionary
% "Learning to Sense Sparse Signals: Simultaneous
% Sensing Matrix and Sparsifying Dictionary
% Optimization"

%min_{Phi,S,Psi,H} \alpha* ||Phi(data - Psi*S)||_F^2/MP + ...
%                \beta* || data - Psi*S||_F^2/NP + ...
%                 +\gamma || Psi' Phi' Phi Psi - H||_F^2
%          s.t. ||S(:,l)||0 \leq \kappa, ||Psi(:,j)||_2 = 1


% Input:
%%%%%%%%%  X_0 - Original signal set of dimension N x P
%%%%%%%%%  Phi_0 - Initial measurement matix of dimension M x N
%%%%%%%%%  Psi_0 - Initial dictionary of dimension N x L
%%%%%%%%%  alpha_dic - A weighting factor in dictionary design
%%%%%%%%%  K - The sparsity
%%%%%%%%%  N_ite - The number of iteration in alternative optimization

% Output:
%%%%%%%%%  Phi_opt - Optimal output measurement matrix of dimension M x N
%%%%%%%%%  Psi_opt - Optimal output dictionary of dimension N x L

function [Phi_opt,Psi_opt]=Simu_Optimization(X_0,Phi_0,Psi_0,K,N_ite,alpha,beta,gamma)

[N,P]=size(X_0);  [M,N2]=size(Phi_0);  [N3,L]=size(Psi_0);  
mu_bd = sqrt((L-N)/(N*(L-1))); % The low bound for mutual coherence
mu_0 = 1*mu_bd;

Phi=Phi_0;
Psi=Psi_0;


alpha = alpha;
beta = beta;
for i=1:N_ite
   %  [Phi]= DCS(Psi,Phi);  % update measurement matrix
  %   Phi =  Closed_DCS(Psi,M);
  H = eye(L);
%     H = (Phi*Psi)'*(Phi*Psi); H([1:L+1:L^2])=0;
%     [index]=find(abs(H)>mu_0);  
%     H(index) = mu_0*sign(H(index));
%     H([1:L+1:L^2]) = 1;   

   nIter = 1; Iter_inside = 1;method = 1;
   [Psi Error]= Simu_Optimization_Dic(X_0,Psi,K,nIter,mu_0,Iter_inside,method,Phi,H,alpha,beta,gamma);

  %  [Phi] = closed_form_SMOSRE(Psi,Error,M,0,Psi'*Psi);
  [Phi] = closed_form_SMOSRE(Psi,Error,M,alpha/gamma,H);
   
 %   [Psi] = Coupled_ISD_Dic_new_image_OMP(X_0,Psi,K,nIter,alpha,mu_0,Iter_inside,method,Phi,alpha_dic);
%   [Psi] = ISD_Dic_new_image_OMP(X_0,Psi,K,nIter,alpha,mu_0,Iter_inside,method);    
 %   [Phi]= DCS(Psi,Phi);  % update measurement matrix  
 %   [Psi]=Coupled_KSVD(X_0,Phi,Psi,K,alpha_dic);  % update dictionary 
end

Phi_opt=Phi;
Psi_opt=Psi;


