function [ Phi_opt ] = closed_form_SMOSRE(Psi,E,M,alpha,Gt)
%==========================================================================
% this function is correspond to the Theroem I if M<N  and Theroem II if
% M>N in our paper to update sensing matrix
%
%Input:    Psi,the given dictionary, N*L
%          E,the traning error matrix
%          M,the dimension of the sensing matrix
%          alpha,the lagrange multipler used in our paper
%
%Output:   Phi_opt,the optimal sensing matrix designed by our proposed
%          algorithm,M*N
%!!!!!!attention please,to avoid the drawback of the "eig" function,we use
%the alternating "svd" function with some manipulations!!!!!!!!!
%
%                        Xiao Li NOV. 2014
%==========================================================================


[N,L] = size(Psi); 

if (nargin<5)
   Gt=eye(L);
end


S = Psi*Gt*Psi' - (alpha/2)*E*E';             
[U_psi,Lambda_psi,V_psi] = svd(Psi);
r = rank(Psi); r = nnz(find(diag(Lambda_psi)>1e-4));
Lambda_psi = Lambda_psi(1:r,1:r);
P_sqr = U_psi(:,1:r)*Lambda_psi*U_psi(:,1:r)';

%================================to avoid the eig function, we use the svd
%function with some manipulations!
Q0 = pinv(P_sqr)*S*pinv(P_sqr);
QQ = Q0*Q0';
[U0,a,b] = svd(QQ);
Lambda = U0'*Q0*U0;
tempLambda = diag(Lambda)';
[tempLambda,Index] = sort(tempLambda,'descend');
Lambda = diag(tempLambda);
%[U0,Lambda] = eig(Q0);  
%================================================

if (M<N)   %this case is correspond to sensing matrix design ie.Theroem I

   Lambda = Lambda(1:M,1:M);
   U0 = U0(:,Index);

   count=0;
for i=1:M
    if( Lambda(i,i)<0 )
       Lambda(i,i) = 0;
       count = count+1;
    end
    if(count==M)
        display('there is no sloution but Phi=0.')
        Phi_opt = 0;
        return;
    end
end

   U = eye(M);
   V0 = eye(N);
   V = U0*V0;
   Sigma = [sqrt(Lambda) zeros(M,N-M)];

   Phi_opt = U*Sigma*V'*pinv(P_sqr);

else        %this following case is correspond to Tansform design ie.Theroem II
    Lambda = Lambda(1:N,1:N);
    U0 = U0(:,Index);

    count=0;
for i=1:N
    if( Lambda(i,i)<0 )
       Lambda(i,i) = 0;
       count = count+1;
    end
    if(count==N)
        display('there is no sloution but Phi=0.')
        Phi_opt = 0;
        return;
    end
end

  U = eye(M);
  V0 = eye(N);
  V = U0*V0;
  Sigma = [sqrt(Lambda) ;zeros(M-N,N)];

  Phi_opt = U*Sigma*V'*pinv(P_sqr);

end

end

